package jalview.ext.ensembl;

import static org.testng.AssertJUnit.assertEquals;
import static org.testng.AssertJUnit.assertFalse;
import static org.testng.AssertJUnit.assertTrue;

import jalview.datamodel.SequenceDummy;
import jalview.datamodel.SequenceFeature;
import jalview.datamodel.SequenceI;
import jalview.io.gff.SequenceOntologyFactory;
import jalview.io.gff.SequenceOntologyLite;
import jalview.util.MapList;

import java.util.List;

import org.testng.Assert;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

public class EnsemblCdsTest
{
  @BeforeClass(alwaysRun = true)
  public void setUp()
  {
    SequenceOntologyFactory.setInstance(new SequenceOntologyLite());
  }

  @AfterClass(alwaysRun = true)
  public void tearDown()
  {
    SequenceOntologyFactory.setInstance(null);
  }

  /**
   * Test that the cdna part of genomic sequence is correctly identified by
   * 'CDS' features (or subtypes) with the desired transcript as parent
   */
  @Test(groups = "Functional")
  public void testGetGenomicRangesFromFeatures()
  {
    EnsemblCds testee = new EnsemblCds();
    SequenceI genomic = new SequenceDummy("chr7");
    genomic.setStart(10000);
    genomic.setEnd(50000);
    String transcriptId = "ABC123";

    // CDS at (start+10000) length 501
    SequenceFeature sf = new SequenceFeature("CDS", "", 20000, 20500, 0f,
            null);
    sf.setValue("Parent", "transcript:" + transcriptId);
    sf.setStrand("+");
    genomic.addSequenceFeature(sf);

    // CDS (sub-type) at (start + 10500) length 101
    sf = new SequenceFeature("CDS_predicted", "", 10500, 10600, 0f, null);
    sf.setValue("Parent", "transcript:" + transcriptId);
    sf.setStrand("+");
    genomic.addSequenceFeature(sf);

    // CDS belonging to a different transcript doesn't count
    sf = new SequenceFeature("CDS", "", 11500, 12600, 0f, null);
    sf.setValue("Parent", "transcript:anotherOne");
    genomic.addSequenceFeature(sf);

    // exon feature doesn't count
    sf = new SequenceFeature("exon", "", 10000, 50000, 0f, null);
    genomic.addSequenceFeature(sf);

    // mRNA_region feature doesn't count (parent of CDS)
    sf = new SequenceFeature("mRNA_region", "", 10000, 50000, 0f, null);
    genomic.addSequenceFeature(sf);

    MapList ranges = testee.getGenomicRangesFromFeatures(genomic,
            transcriptId, 23);
    List<int[]> fromRanges = ranges.getFromRanges();
    assertEquals(2, fromRanges.size());
    // from ranges should be sorted by start order
    assertEquals(10500, fromRanges.get(0)[0]);
    assertEquals(10600, fromRanges.get(0)[1]);
    assertEquals(20000, fromRanges.get(1)[0]);
    assertEquals(20500, fromRanges.get(1)[1]);
    // to range should start from given start numbering
    List<int[]> toRanges = ranges.getToRanges();
    assertEquals(1, toRanges.size());
    assertEquals(23, toRanges.get(0)[0]);
    assertEquals(624, toRanges.get(0)[1]);
  }

  /**
   * Test the method that retains features except for 'CDS' (or subtypes), or
   * features with parent other than the given id
   */
  @Test(groups = "Functional")
  public void testRetainFeature()
  {
    String accId = "ABC123";
    EnsemblCds testee = new EnsemblCds();

    SequenceFeature sf = new SequenceFeature("CDS", "", 20000, 20500, 0f,
            null);
    assertFalse(testee.retainFeature(sf, accId));

    sf.setType("CDS_predicted");
    assertFalse(testee.retainFeature(sf, accId));

    // other feature with no parent is retained
    sf.setType("sequence_variant");
    assertTrue(testee.retainFeature(sf, accId));

    // other feature with desired parent is retained
    sf.setValue("Parent", "transcript:" + accId);
    assertTrue(testee.retainFeature(sf, accId));

    // feature with wrong parent is not retained
    sf.setValue("Parent", "transcript:XYZ");
    assertFalse(testee.retainFeature(sf, accId));
  }

  /**
   * Test the method that picks out 'CDS' (or subtype) features with the
   * accession id as parent
   */
  @Test(groups = "Functional")
  public void testIdentifiesSequence()
  {
    String accId = "ABC123";
    EnsemblCds testee = new EnsemblCds();

    // cds with no parent not valid
    SequenceFeature sf = new SequenceFeature("CDS", "", 1, 2, 0f, null);
    assertFalse(testee.identifiesSequence(sf, accId));

    // cds with wrong parent not valid
    sf.setValue("Parent", "transcript:XYZ");
    assertFalse(testee.identifiesSequence(sf, accId));

    // cds with right parent is valid
    sf.setValue("Parent", "transcript:" + accId);
    assertTrue(testee.identifiesSequence(sf, accId));

    // cds sub-type with right parent is valid
    sf.setType("CDS_predicted");
    assertTrue(testee.identifiesSequence(sf, accId));

    // transcript not valid:
    sf.setType("transcript");
    assertFalse(testee.identifiesSequence(sf, accId));

    // exon not valid:
    sf.setType("exon");
    assertFalse(testee.identifiesSequence(sf, accId));
  }

  @Test(groups = "Functional")
  public void testIsValidReference() throws Exception
  {
    EnsemblSequenceFetcher esq = new EnsemblCds();
    Assert.assertTrue(esq.isValidReference("CCDS5863.1"));
    Assert.assertTrue(esq.isValidReference("ENST00000288602"));
    Assert.assertTrue(esq.isValidReference("ENSG00000288602"));
    Assert.assertTrue(esq.isValidReference("ENSP00000288602"));
    Assert.assertFalse(esq.isValidReference("ENST0000288602"));
    // non-human species have a 3 character identifier included:
    Assert.assertTrue(esq.isValidReference("ENSMUSG00000099398"));
  }

}
